@@ -449,6 +449,7 @@ class DBM:
       - max_page_size (int): The maximum size of a page.
       - max_branches (int): The maximum number of branches each inner node can have.
       - max_cached_pages (int): The maximum number of cached pages.
+      - page_update_mode (string): What to do when each page is updated: "PAGE_UPDATE_NONE" is to do no operation or "PAGE_UPDATE_WRITE" is to write immediately.
       - key_comparator (string): The comparator of record keys: "LexicalKeyComparator" for the lexical order, "LexicalCaseKeyComparator" for the lexical order ignoring case, "DecimalKeyComparator" for the order of the decimal integer numeric expressions, "HexadecimalKeyComparator" for the order of the hexadecimal integer numeric expressions, "RealNumberKeyComparator" for the order of the decimal real number expressions.
 
     For SkipDBM, these optional parameters are supported.
@@ -492,6 +493,19 @@ class DBM:
     """
     pass  # native code
 
+  def Process(self, key, func, writable):
+    """
+    Processes a record with an arbitrary function.
+
+    :param key: The key of the record.
+    :param func: The function to process a record.  The first parameter is the key of the record.  The second parameter is the value of the existing record, or None if it the record doesn't exist.  The return value is a string or bytes to update the record value.  If the return value is None, the record is not modified.  If the return value is False (not a false value but the False object), the record is removed.
+    :param writable: True if the processor can edit the record.
+    :return: The result status.
+
+    This method is not available in the concurrent mode because the function cannot be invoked outside the GIL.
+    """
+    pass  # native code
+
   def Get(self, key, status=None):
     """
     Gets the value of a record of a key.
@@ -705,6 +719,18 @@ class DBM:
     """
     pass  # native code
 
+  def ProcessEach(self, func, writable):
+    """
+    Processes each and every record in the database with an arbitrary function.
+
+    :param func: The function to process a record.  The first parameter is the key of the record.  The second parameter is the value of the existing record, or None if it the record doesn't exist.  The return value is a string or bytes to update the record value.  If the return value is None, the record is not modified.  If the return value is False (not a false value but the False object), the record is removed.
+    :param writable: True if the processor can edit the record.
+    :return: The result status.
+
+    The given function is called repeatedly for each record.  It is also called once before the iteration and once after the iteration with both the key and the value being None.  This method is not available in the concurrent mode because the function cannot be invoked outside the GIL.
+    """
+    pass  # native code
+
   def Count(self):
     """
     Gets the number of records.
@@ -874,7 +900,7 @@ class DBM:
     """
     Searches the database and get keys which match a pattern.
 
-    :param mode: The search mode.  "contain" extracts keys containing the pattern.  "begin" extracts keys beginning with the pattern.  "end" extracts keys ending with the pattern.  "regex" extracts keys partially matches the pattern of a regular expression.  "edit" extracts keys whose edit distance to the UTF-8 pattern is the least.  "editbin" extracts keys whose edit distance to the binary pattern is the least.
+    :param mode: The search mode.  "contain" extracts keys containing the pattern.  "begin" extracts keys beginning with the pattern.  "end" extracts keys ending with the pattern.  "regex" extracts keys partially matches the pattern of a regular expression.  "edit" extracts keys whose edit distance to the UTF-8 pattern is the least.  "editbin" extracts keys whose edit distance to the binary pattern is the least.  "containcase", "containword", and "containcaseword" extract keys considering case and word boundary.  Ordered databases support "upper" and "lower" which extract keys whose positions are upper/lower than the pattern. "upperinc" and "lowerinc" are their inclusive versions.
     :param pattern: The pattern for matching.
     :param capacity: The maximum records to obtain.  0 means unlimited.
     :return: A list of string keys matching the condition.
@@ -1249,6 +1275,18 @@ class AsyncDBM:
     """
     pass  # native code
 
+  def ProcessMulti(self, key_func_pairs, writable):
+    """
+    Processes multiple records with arbitrary functions.
+
+    :param key_func_pairs: A list of pairs of keys and their functions.  The first parameter of the function is the key of the record.  The second parameter is the value of the existing record, or None if it the record doesn't exist.  The return value is a string or bytes to update the record value.  If the return value is None, the record is not modified.  If the return value is False (not a false value but the False object), the record is removed.
+    :param writable: True if the processors can edit the record.
+    :return: The result status.
+
+    This method is not available in the concurrent mode because the function cannot be invoked outside the GIL.
+    """
+    pass  # native code
+
   def CompareExchangeMulti(self, expected, desired):
     """
     Compares the values of records and exchanges if the condition meets.
